function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

/** @jsx h */
import Preact, { h, Component } from 'preact';

var List = function (_Component) {
    _inherits(List, _Component);

    function List(props) {
        _classCallCheck(this, List);

        var _this = _possibleConstructorReturn(this, _Component.call(this, props));

        _this.state = {
            startIndex: 0,
            sizePerPage: 1
        };
        _this.update = _this.update.bind(_this);
        _this.itemHeight = 0;
        _this.scrollContainer = null;
        _this.listContainer = null;
        return _this;
    }

    List.prototype.componentDidMount = function componentDidMount() {
        var items = this.items.children || [];
        this.itemHeight = items[0] && items[0]['offsetHeight'];
        this.scrollContainer = this.getScroller();
        this.scrollContainer.addEventListener('scroll', this.update);
        window.addEventListener('resize', this.update);
        this.update();
    };

    List.prototype.componentDidUpdate = function componentDidUpdate() {
        this.update();
    };

    List.prototype.componentWillUnmount = function componentWillUnmount() {
        this.scrollContainer.removeEventListener('scroll', this.update);
        window.removeEventListener('resize', this.update);
    };

    List.prototype.getScroller = function getScroller() {
        var overflowReg = /(auto|scroll)/;
        for (var style, el = this.listContainer; el = el.parentElement;) {
            style = window.getComputedStyle(el);
            if (overflowReg.test(style.overflow + style.overflowY)) return el;
        }
        return window;
    };

    List.prototype.calculate = function calculate() {
        var length = this.props.length,
            itemHeight = this.itemHeight;

        var scrollDistance = this.getScrollDistance();
        var top = Math.max(0, scrollDistance - 100),
            bottom = Math.min(scrollDistance + this.getViewportHeight() + 100, this.getDistance(length));

        var sizePerPage = Math.min(Math.max(Math.ceil((bottom - top) / itemHeight) + 1, 1), length),
            startIndex = Math.min(Math.floor(top / itemHeight), length - sizePerPage);

        return { startIndex: startIndex, sizePerPage: sizePerPage };
    };

    List.prototype.update = function update() {
        var _calculate = this.calculate(),
            startIndex = _calculate.startIndex,
            sizePerPage = _calculate.sizePerPage;

        if (startIndex !== this.state.startIndex || sizePerPage !== this.state.sizePerPage) this.setState({ startIndex: startIndex, sizePerPage: sizePerPage });
    };

    List.prototype.getOffset = function getOffset(el) {
        var offset = el['clientTop'] || 0;
        do {
            offset += el['offsetTop'] || 0;
        } while (el = el.offsetParent);
        return offset;
    };

    List.prototype.getDistance = function getDistance(index) {
        return index * this.itemHeight;
    };

    List.prototype.getScrollHeight = function getScrollHeight() {
        var scrollContainer = this.scrollContainer;
        return scrollContainer === window ? Math.max(document.body['scrollHeight'], document.documentElement['scrollHeight']) : scrollContainer['scrollHeight'];
    };

    List.prototype.getViewportHeight = function getViewportHeight() {
        var scrollContainer = this.scrollContainer;
        return scrollContainer === window ? window['innerHeight'] : scrollContainer['clientHeight'];
    };

    List.prototype.getScrollDistance = function getScrollDistance() {
        var scrollContainer = this.scrollContainer,
            listContainer = this.listContainer,
            max = this.getScrollHeight() - this.getViewportHeight(),
            scrollTop = scrollContainer === window ? document.body['scrollTop'] || document.documentElement['scrollTop'] : scrollContainer['scrollTop'],
            scrollDistance = Math.max(0, Math.min(scrollTop, max));
        return this.getOffset(scrollContainer) + scrollDistance - this.getOffset(listContainer);
    };

    List.prototype.getVisibleItems = function getVisibleItems(startIndex, sizePerPage) {
        var items = [];
        for (var i = 0; i < sizePerPage; ++i) {
            items.push(startIndex + i);
        }
        return items;
    };

    List.prototype.getContentStyle = function getContentStyle(startIndex) {
        var offset = this.getDistance(startIndex),
            transform = 'translate(0px, ' + offset + 'px)';
        return {
            msTransform: transform,
            WebkitTransform: transform,
            transform: transform
        };
    };

    List.prototype.render = function render() {
        var _this2 = this;

        var _props = this.props,
            renderItem = _props.renderItem,
            length = _props.length;
        var _state = this.state,
            startIndex = _state.startIndex,
            sizePerPage = _state.sizePerPage;


        var visibleItems = this.getVisibleItems(startIndex, sizePerPage),
            visibleItemComps = visibleItems.map(renderItem),
            contentStyle = this.getContentStyle(startIndex);

        return h(
            'div',
            { style: {
                    position: 'relative',
                    height: length * this.itemHeight
                }, ref: function ref(c) {
                    return _this2.listContainer = c;
                } },
            h(
                'div',
                { ref: function ref(d) {
                        return _this2.items = d;
                    },
                    style: contentStyle },
                visibleItemComps
            )
        );
    };

    return List;
}(Component);

export { List as default };