import React, { Component } from 'react';

export default class List extends Component {
    constructor(props) {
        super(props);
        this.state = {
            startIndex: 0,
            sizePerPage: 1,
        };
        this.update = this.update.bind(this);
        this.itemHeight = 0;
        this.scrollContainer = null;
        this.listContainer = null;
    }

    componentDidMount() {
        const items = this.items.children || [];
        this.itemHeight = items[0] && items[0]['offsetHeight'];
        this.scrollContainer = this.getScroller();
        this.scrollContainer.addEventListener('scroll', this.update);
        window.addEventListener('resize', this.update);
        this.update();
    }

    componentDidUpdate() {
        this.update();
    }

    componentWillUnmount() {
        this.scrollContainer.removeEventListener('scroll', this.update);
        window.removeEventListener('resize', this.update);
    }

    getScroller() {
        let overflowReg = /(auto|scroll)/;
        for (let style, el = this.listContainer; el = el.parentElement;) {
            style = window.getComputedStyle(el);
            if (overflowReg.test(style.overflow + style.overflowY)) return el;
        }
        return window;
    }

    calculate() {
        let length = this.props.length,
            itemHeight = this.itemHeight;

        let scrollDistance = this.getScrollDistance();
        let top = Math.max(0, scrollDistance - 100),
            bottom = Math.min(scrollDistance + this.getViewportHeight() + 100, this.getDistance(length));

        let sizePerPage = Math.min(Math.max(Math.ceil((bottom - top) / itemHeight) + 1, 1), length),
            startIndex = Math.min(Math.floor(top / itemHeight), length - sizePerPage);

        return { startIndex, sizePerPage };
    }

    update() {
        const { startIndex, sizePerPage } = this.calculate();
        if (startIndex !== this.state.startIndex || sizePerPage !== this.state.sizePerPage) this.setState({ startIndex, sizePerPage });
    }
    
    getOffset(el) {
        let offset = el['clientTop'] || 0;
        do offset += el['offsetTop'] || 0; while (el = el.offsetParent);
        return offset;
    }

    getDistance(index) {
        return index * this.itemHeight;
    }

    getScrollHeight() {
        const scrollContainer = this.scrollContainer;
        return scrollContainer === window ?
            Math.max(document.body['scrollHeight'], document.documentElement['scrollHeight']) :
            scrollContainer['scrollHeight'];
    }

    getViewportHeight() {
        const scrollContainer = this.scrollContainer;
        return scrollContainer === window ? window['innerHeight'] : scrollContainer['clientHeight'];
    }

    getScrollDistance() {
        const scrollContainer = this.scrollContainer,
            listContainer = this.listContainer,
            max = this.getScrollHeight() - this.getViewportHeight(),
            scrollTop = scrollContainer === window ?
                document.body['scrollTop'] || document.documentElement['scrollTop'] :
                scrollContainer['scrollTop'],
            scrollDistance = Math.max(0, Math.min(scrollTop, max));
        return this.getOffset(scrollContainer) + scrollDistance - this.getOffset(listContainer);
    }

    getVisibleItems(startIndex, sizePerPage) {
        const items = [];
        for (let i = 0; i < sizePerPage; ++i) {
            items.push(startIndex + i);
        }
        return items;
    }

    getContentStyle(startIndex) {
        const offset = this.getDistance(startIndex),
            transform = `translate(0px, ${offset}px)`;
        return {
            msTransform: transform,
            WebkitTransform: transform,
            transform
        };
    }

    render() {
        const { renderItem, length } = this.props;
        const { startIndex, sizePerPage } = this.state;
        
        let visibleItems = this.getVisibleItems(startIndex, sizePerPage),
            visibleItemComps = visibleItems.map(renderItem),
            contentStyle = this.getContentStyle(startIndex);

        return (
            <div style={{
                position: 'relative',
                height: (length * this.itemHeight)
            }} ref={c => this.listContainer = c}>
                <div ref={d => this.items = d}
                    style={contentStyle}>{visibleItemComps}</div>
            </div>
        );
    }
}